// Copyright 2024 Brian Newman. All rights reserved.

// Package response ...
package response

import (
	"encoding/json"
)

// Return codes
const (
	OK    = "200"
	ERROR = "500"
)

// New ...
func New(code string, message string, data []byte) *Response {
	return &Response{
		Code:    code,
		Message: message,
		Data:    data,
	}
}

// Response ...
type Response struct {
	Code    string `json:"code"`
	Message string `json:"message"`
	Data    []byte `json:"json"`
}

// ToJSON ...
func (x *Response) ToJSON() (data []byte, err error) {
	data, err = json.Marshal(x)
	return
}

// Parse ...
func (x *Response) Parse(data []byte) (err error) {
	err = json.Unmarshal(data, x)
	return
}
